"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTestCLI = exports.testJatsFile = void 0;
const commander_1 = require("commander");
const myst_cli_utils_1 = require("myst-cli-utils");
const session_1 = require("../session");
const js_yaml_1 = __importDefault(require("js-yaml"));
const fs_1 = __importDefault(require("fs"));
const parse_1 = require("./parse");
const unist_util_select_1 = require("unist-util-select");
const unist_util_is_1 = require("unist-util-is");
const chalk_1 = __importDefault(require("chalk"));
const INDENT = '       ';
function printNodes(expected, received) {
    return chalk_1.default.reset(`\n${INDENT}${chalk_1.default.greenBright('Expected node containing')}:\n${INDENT}  ${js_yaml_1.default
        .dump(expected)
        .replace(/\n/g, `\n${INDENT}  `)}\n${INDENT}${chalk_1.default.redBright('Received node')}:\n${INDENT}  ${js_yaml_1.default.dump(received).replace(/\n/g, `\n${INDENT}  `)}`);
}
function testJatsFile(session, file, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const toc = (0, myst_cli_utils_1.tic)();
        const jats = yield (0, parse_1.parseJats)(session, file);
        const tests = js_yaml_1.default.load(fs_1.default.readFileSync(opts.cases).toString());
        const results = tests.cases.map((testCase, index) => {
            if (!testCase.title) {
                return [`Test Case ${index}`, null, 'Test must include a title'];
            }
            if (testCase.equals === undefined) {
                return [testCase.title, null, 'Test must have an equals statement'];
            }
            if (testCase.select) {
                const node = (0, unist_util_select_1.select)(testCase.select, jats.tree);
                const pass = (0, unist_util_is_1.is)(node, testCase.equals);
                if (testCase.equals == null && node) {
                    return [testCase.title, false, 'Expected no node to be present'];
                }
                if (!node && testCase.equals == null)
                    return [testCase.title, true];
                if (!node)
                    return [testCase.title, false];
                let failed = false;
                const messages = [];
                if (!pass) {
                    failed = failed || true;
                    messages.push(`Failed to validate node\n${printNodes(testCase.equals, node)}`);
                }
                return [testCase.title, !failed, messages.join('\n')];
            }
            else if (testCase.selectAll) {
                const testNodes = (0, unist_util_select_1.selectAll)(testCase.selectAll, jats.tree);
                if (!testNodes && testCase.equals == null)
                    return [testCase.title, true];
                if (!testNodes)
                    return [testCase.title, false, 'Node not found'];
                let equals = testCase.equals;
                if (!Array.isArray(testCase.equals)) {
                    equals = Array(testNodes.length).fill(testCase.equals);
                }
                let failed = false;
                const messages = [];
                if (equals.length !== testNodes.length) {
                    failed = failed || true;
                    messages.push(`Expected ${equals.length} nodes, got ${testNodes.length}\n${printNodes(equals, testNodes)}`);
                }
                else {
                    equals.forEach((node, ii) => {
                        const pass = (0, unist_util_is_1.is)(testNodes[ii], node);
                        if (!pass) {
                            failed = failed || true;
                            messages.push(`Failed to validate node ${ii}\n${printNodes(node, testNodes[ii])}`);
                        }
                    });
                }
                return [testCase.title, !failed, messages.join('\n')];
            }
            else {
                return [testCase.title, false, 'Test must have either `select` or `selectAll`'];
            }
        });
        results.forEach((result) => {
            const [title, pass, message] = result;
            if (pass === null)
                session.log.info(`${chalk_1.default.redBright.bold(`ERROR`)} - ${title}\n  ${chalk_1.default.blueBright(message)}`);
            else if (pass)
                session.log.info(`${chalk_1.default.green(`PASS`)} - ${title}`);
            else
                session.log.info(`${chalk_1.default.red(`FAIL`)} - ${title}\n\n${INDENT}${chalk_1.default.blueBright(message)}\n`);
        }, true);
        const passed = results.reduce((num, [, pass]) => num + (pass ? 1 : 0), 0);
        const failed = results.length - passed;
        if (failed > 0 && passed === 0) {
            throw new Error(toc(`${chalk_1.default.red(`Failed ${failed} tests in %s`)} 👎`));
        }
        if (failed > 0) {
            throw new Error(toc(`${chalk_1.default.green(`Passed ${passed}/${results.length} tests in %s`)}\n${chalk_1.default.red(`Failed ${failed} tests`)} 👎`));
        }
        session.log.info(chalk_1.default.green(toc(`Passed ${passed} tests in %s 🚀`)));
        return true;
    });
}
exports.testJatsFile = testJatsFile;
function makeTestCLI(program) {
    const command = new commander_1.Command('test')
        .description('Test JATS file against a list of cases')
        .argument('<file>', 'JATS file to test')
        .addOption(new commander_1.Option('--cases <value>', 'The YAML file of unit tests to test against'))
        .action((0, myst_cli_utils_1.clirun)(testJatsFile, { program, getSession: session_1.getSession }));
    return command;
}
function addTestCLI(program) {
    program.addCommand(makeTestCLI(program));
}
exports.addTestCLI = addTestCLI;
