"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Jats = void 0;
const myst_common_1 = require("myst-common");
const xml_js_1 = require("xml-js");
const doi_utils_1 = __importDefault(require("doi-utils"));
const utils_1 = require("./utils");
const unist_util_select_1 = require("unist-util-select");
const jats_tags_1 = require("jats-tags");
const myst_cli_utils_1 = require("myst-cli-utils");
function select(selector, node) {
    var _a;
    return ((_a = (0, unist_util_select_1.select)(selector, node)) !== null && _a !== void 0 ? _a : undefined);
}
class Jats {
    constructor(data, opts) {
        var _a;
        const toc = (0, myst_cli_utils_1.tic)();
        this.log = opts === null || opts === void 0 ? void 0 : opts.log;
        if (opts === null || opts === void 0 ? void 0 : opts.source)
            this.source = opts.source;
        try {
            this.raw = (0, xml_js_1.xml2js)(data, { compact: false });
        }
        catch (error) {
            throw new Error('Problem parsing the JATS document, please ensure it is XML');
        }
        const { declaration, elements } = this.raw;
        this.declaration = declaration === null || declaration === void 0 ? void 0 : declaration.attributes;
        if (!((elements === null || elements === void 0 ? void 0 : elements.length) === 2 && elements[0].type === 'doctype' && hasSingleArticle(elements[1]))) {
            throw new Error('Element <article> is not the only element of the JATS');
        }
        this.doctype = elements[0].doctype;
        const converted = (0, utils_1.convertToUnist)(elements[1]);
        this.tree = select('article', converted);
        (_a = this.log) === null || _a === void 0 ? void 0 : _a.debug(toc('Parsed and converted JATS to unist tree in %s'));
    }
    get frontmatter() {
        var _a, _b, _c, _d;
        const title = this.articleTitle;
        const subtitle = this.articleSubtitle;
        const date = this.publicationDate;
        const authors = this.articleAuthors;
        const firstSubject = select(jats_tags_1.Tags.subject, (_a = this.articleCategories) !== null && _a !== void 0 ? _a : this.front);
        const journalTitle = select(jats_tags_1.Tags.journalTitle, this.front);
        return {
            title: title ? (0, myst_common_1.toText)(title) : undefined,
            subtitle: subtitle ? (0, myst_common_1.toText)(subtitle) : undefined,
            doi: (_b = this.doi) !== null && _b !== void 0 ? _b : undefined,
            date: date ? (_c = (0, utils_1.toDate)(date)) === null || _c === void 0 ? void 0 : _c.toISOString() : undefined,
            authors: authors === null || authors === void 0 ? void 0 : authors.map((a) => (0, utils_1.authorAndAffiliation)(a, this.tree)),
            keywords: (_d = this.keywords) === null || _d === void 0 ? void 0 : _d.map((k) => (0, myst_common_1.toText)(k)),
            venue: journalTitle ? { title: (0, myst_common_1.toText)(journalTitle) } : undefined,
            subject: firstSubject ? (0, myst_common_1.toText)(firstSubject) : undefined,
        };
    }
    get front() {
        return select(jats_tags_1.Tags.front, this.tree);
    }
    get premissions() {
        return select(jats_tags_1.Tags.permissions, this.front);
    }
    get doi() {
        var _a;
        return doi_utils_1.default.normalize((_a = (0, utils_1.findArticleId)(this.front, 'doi')) !== null && _a !== void 0 ? _a : '');
    }
    get pmc() {
        var _a;
        return (_a = (0, utils_1.findArticleId)(this.front, 'pmc')) === null || _a === void 0 ? void 0 : _a.replace(/^PMC:?/, '');
    }
    get pmid() {
        return (0, utils_1.findArticleId)(this.front, 'pmid');
    }
    get publicationDates() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.pubDate, this.front);
    }
    get publicationDate() {
        return this.publicationDates.find((d) => !!select(jats_tags_1.Tags.day, d));
    }
    get license() {
        return select(jats_tags_1.Tags.license, this.premissions);
    }
    get keywordGroup() {
        return select(jats_tags_1.Tags.kwdGroup, this.front);
    }
    /** The first keywords */
    get keywords() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.kwd, this.keywordGroup);
    }
    get keywordGroups() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.kwdGroup, this.front);
    }
    get articleCategories() {
        return select(jats_tags_1.Tags.articleCategories, this.front);
    }
    get titleGroup() {
        return select(jats_tags_1.Tags.titleGroup, this.front);
    }
    get articleTitle() {
        return select(jats_tags_1.Tags.articleTitle, this.titleGroup);
    }
    get articleSubtitle() {
        return select(jats_tags_1.Tags.subtitle, this.titleGroup);
    }
    get abstract() {
        return select(jats_tags_1.Tags.abstract, this.front);
    }
    get abstracts() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.abstract, this.front);
    }
    get contribGroup() {
        return select(jats_tags_1.Tags.contribGroup, this.front);
    }
    get contribGroups() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.contribGroup, this.front);
    }
    get articleAuthors() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.contrib, this.contribGroup);
    }
    get body() {
        return select(jats_tags_1.Tags.body, this.tree);
    }
    get back() {
        return select(jats_tags_1.Tags.back, this.tree);
    }
    get subArticles() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.subArticle, this.tree);
    }
    get refList() {
        return select(jats_tags_1.Tags.refList, this.back);
    }
    get references() {
        return (0, unist_util_select_1.selectAll)(jats_tags_1.Tags.ref, this.refList);
    }
}
exports.Jats = Jats;
function hasSingleArticle(element) {
    var _a;
    if (element.name === 'article') {
        return true;
    }
    if (element.name === 'pmc-articleset' &&
        ((_a = element.elements) === null || _a === void 0 ? void 0 : _a.length) === 1 &&
        element.elements[0].name === 'article') {
        return true;
    }
    return false;
}
