"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.authorAndAffiliation = exports.findArticleId = exports.formatDate = exports.toDate = exports.convertToUnist = void 0;
const myst_common_1 = require("myst-common");
const doi_utils_1 = __importDefault(require("doi-utils"));
const unist_util_select_1 = require("unist-util-select");
const jats_tags_1 = require("jats-tags");
function convertToUnist(node) {
    switch (node.type) {
        case 'element': {
            const { name, attributes, elements } = node;
            const children = elements === null || elements === void 0 ? void 0 : elements.map(convertToUnist).filter((n) => !!n);
            const next = Object.assign({ type: name !== null && name !== void 0 ? name : 'unknown' }, attributes);
            if (name === 'code') {
                next.value = elements === null || elements === void 0 ? void 0 : elements[0].text;
            }
            else if (children)
                next.children = children;
            return next;
        }
        case 'text': {
            const { attributes, text } = node;
            return Object.assign(Object.assign({ type: 'text' }, attributes), { value: String(text)
                    .replace(/\n(\s+)$/, '')
                    .replace('\n', ' ') });
        }
        case 'cdata': {
            const { attributes, cdata } = node;
            return Object.assign(Object.assign({ type: 'cdata' }, attributes), { cdata: String(cdata).trim() });
        }
        case 'comment': {
            const { comment } = node;
            return { type: 'comment', value: String(comment) };
        }
        case 'instruction': {
            // For example:
            // <?properties manuscript?> becomes:
            // { type: 'instruction', name: 'properties', instruction: 'manuscript' }
            return undefined;
        }
        default:
            console.log(node);
            throw new Error(`found ${node.type} ${node.name}`);
    }
}
exports.convertToUnist = convertToUnist;
const MonthLookup = {
    jan: 0,
    january: 0,
    feb: 1,
    february: 1,
    mar: 2,
    march: 2,
    apr: 3,
    april: 3,
    may: 4,
    jun: 5,
    june: 5,
    jul: 6,
    july: 6,
    aug: 7,
    august: 7,
    sep: 8,
    sept: 8,
    september: 8,
    oct: 9,
    october: 9,
    nov: 10,
    november: 10,
    dec: 11,
    december: 11,
};
function toDate(date) {
    if (!date)
        return;
    const isoDate = date['iso-8601-date'];
    if (isoDate)
        return new Date(isoDate);
    const year = Number((0, myst_common_1.toText)((0, unist_util_select_1.select)('year', date)));
    if (!year || Number.isNaN(year))
        return;
    const monthText = (0, myst_common_1.toText)((0, unist_util_select_1.select)('month', date));
    const monthTextNumber = Number(monthText);
    const month = Number.isNaN(monthTextNumber) ? MonthLookup[monthText] : monthTextNumber - 1;
    if (month == null)
        return new Date(Date.UTC(year, 0));
    const day = Number((0, myst_common_1.toText)((0, unist_util_select_1.select)('day', date)));
    if (!day || Number.isNaN(day))
        return new Date(Date.UTC(year, month));
    return new Date(Date.UTC(year, month, day));
}
exports.toDate = toDate;
function formatDate(date) {
    if (!date)
        return;
    return new Intl.DateTimeFormat('en-US', { dateStyle: 'long', timeZone: 'UTC' }).format(date);
}
exports.formatDate = formatDate;
function findArticleId(node, pubIdType = 'doi') {
    if (!node)
        return undefined;
    const id = (0, unist_util_select_1.select)(`[pub-id-type=${pubIdType}]`, node);
    if (id && (0, myst_common_1.toText)(id))
        return (0, myst_common_1.toText)(id);
    const doiTag = (0, unist_util_select_1.selectAll)(`${jats_tags_1.Tags.articleId},${jats_tags_1.Tags.pubId}`, node).find((t) => doi_utils_1.default.validate((0, myst_common_1.toText)(t)));
    return (0, myst_common_1.toText)(doiTag) || undefined;
}
exports.findArticleId = findArticleId;
function authorAndAffiliation(node, article) {
    const author = {
        name: `${(0, myst_common_1.toText)((0, unist_util_select_1.select)(jats_tags_1.Tags.givenNames, node))} ${(0, myst_common_1.toText)((0, unist_util_select_1.select)(jats_tags_1.Tags.surname, node))}`,
    };
    const orcid = (0, unist_util_select_1.select)('[contrib-id-type=orcid]', node);
    if (orcid) {
        author.orcid = (0, myst_common_1.toText)(orcid).replace(/(https?:\/\/)?orcid\.org\//, '');
    }
    //
    /**
     * For example:
     *
     * ```xml
     * <aff id="aff2">
     * <label>2</label>
     * <institution-wrap>
     * <institution-id institution-id-type="ror">https://ror.org/00t9vx427</institution-id>
     * <institution>Department of Biochemistry, University of Texas Southwestern Medical Center</institution>
     * </institution-wrap>
     * <addr-line>
     * <named-content content-type="city">Dallas</named-content>
     * </addr-line>
     * <country>United States</country>
     * </aff>
     * ```
     */
    const affiliationRefs = (0, unist_util_select_1.selectAll)('xref[ref-type=aff]', node);
    const affiliations = affiliationRefs.map((xref) => (0, unist_util_select_1.select)(`[id=${xref.rid}]`, article));
    const affiliationText = affiliations
        .map((aff) => {
        // TODO: handle rors!
        const ror = (0, unist_util_select_1.select)(`[institution-id-type=ror]`, aff);
        return (0, myst_common_1.toText)((0, unist_util_select_1.select)('institution', aff));
    })
        .filter((t) => !!t);
    if (affiliationText.length > 0) {
        author.affiliations = affiliationText;
    }
    return author;
}
exports.authorAndAffiliation = authorAndAffiliation;
